using System;
using System.Collections.Specialized;

namespace InSiteTransactionProfiler
{
	public class Exec
	{
		public DateTime startTime;
		public int processID;
		public string execName;
		public StringCollection paramName = new StringCollection();
		public StringCollection paramValue = new StringCollection();
		public Exec(string exec, DateTime startTime, int processID)
		{
			this.execName = exec;
			this.startTime = startTime;
			this.processID = processID;
		}
	}

	/// <summary>
	/// CDOInfo Base Class
	/// </summary>
	public abstract class CDOInfo
	{
		public DateTime startTime;
		public int processID;
		public int depth;
		public string CDOname;
		public ExecList execs = new ExecList();
		public CDOInfo(DateTime startTime, int processID, int depth, string cdoName)
		{
			this.startTime = startTime;
			this.processID = processID;
			this.depth = depth;
			this.CDOname = cdoName;
		}
	}


	/// <summary>
	/// log start at: "Depth: ? CDO ? Event: ?" (No colon after CDO)
	/// </summary>
	public class CDOEvent : CDOInfo
	{
		public string eventName;
		public CDOEvent(string eventName, DateTime startTime, int processID, int depth, string cdoName) : base(startTime,processID, depth, cdoName)
		{
			this.eventName = eventName;
		}
	}


	/// <summary>
	/// log start at: "Depth: ? CDO: ? Perform: ?" (A colon after CDO)
	/// </summary>
	public class CDOPerform : CDOInfo
	{
		public string performName;
		public CDOPerform(string perform, DateTime startTime, int processID, int depth, string cdoName) : base(startTime,processID, depth, cdoName)
		{
			this.performName = perform;
		}
	}
	
	/// <summary>
	/// log start at: "Depth: ? CDO: ? Field: ? Event: ?" (A colon after CDO)
	/// </summary>
	public class CDOFieldEvent : CDOInfo
	{
		public string eventName;
		public string fieldName;
		public CDOFieldEvent(string eventName, string fieldName, DateTime startTime, int processID, int depth, string cdoName) : base(startTime,processID, depth, cdoName)
		{
			this.eventName = eventName;
			this.fieldName = fieldName;
		}
	}

	#region InSiteTxnLog is a safe ArrayList of class CDOInfo which contain all information of an InSite Transaction Log
	public class InSiteTxnLog : System.Collections.CollectionBase  
	{
		public DateTime service_start_time;
		public int service_start_pcs_id;
		public DateTime service_end_time;
		public int service_end_pcs_id;
		public string service_name;
		public string user_name;

		public DateTime LastCDOEndTime
		{
			get
			{ 
				int lastCDO, lastExec;
				lastCDO = List.Count - 1;
				if( this[lastCDO].execs.Count == 0 )
				{
					lastExec =  0;
					lastCDO --;
				}
				else
					lastExec =  this[lastCDO].execs.Count-1;
				return this[lastCDO].execs[lastExec].startTime;
			}
		}

		public CDOInfo this[ int index ]  
		{
			get  
			{
				return( (CDOInfo) List[index] );
			}
			set  
			{
				List[index] = value;
			}
		}

		public int Add( CDOInfo value )  
		{
			return( List.Add( value ) );
		}

		public int IndexOf( CDOInfo value )  
		{
			return( List.IndexOf( value ) );
		}

		public void Insert( int index, CDOInfo value )  
		{
			List.Insert( index, value );
		}

		public void Remove( CDOInfo value )  
		{
			List.Remove( value );
		}

		public bool Contains( CDOInfo value )  
		{
			// If value is not of type CDOInfo, this will return false.
			return( List.Contains( value ) );
		}
	}
	#endregion

	
	#region ExecList is a type safe ArrayList of class Exec
	public class ExecList : System.Collections.CollectionBase  
	{

		public Exec this[ int index ]  
		{
			get  
			{
				return( (Exec) List[index] );
			}
			set  
			{
				List[index] = value;
			}
		}

		public int Add( Exec value )  
		{
			return( List.Add( value ) );
		}

		public int IndexOf( Exec value )  
		{
			return( List.IndexOf( value ) );
		}

		public void Insert( int index, Exec value )  
		{
			List.Insert( index, value );
		}

		public void Remove( Exec value )  
		{
			List.Remove( value );
		}

		public bool Contains( Exec value )  
		{
			// If value is not of type Exec, this will return false.
			return( List.Contains( value ) );
		}

		protected override void OnValidate( Object value )  
		{
			if ( value.GetType() != Type.GetType("InSiteTransactionProfiler.Exec") )
				throw new ArgumentException( "value must be of type Exec.", "value" );
		}
	}
	#endregion
	

}
